----------------------------------------------------------------------------------------
--   Script:		MAGMApulators:Geometry Manipulators 
--	 Version:		v0.40b
--	 Started:       10/29/00
--	 Last Modified: 01/29/00
--   Description:	Adds Manipulators for Height, Width, Length, Radius1, Radius2,
--				 	CapHeight/Fillet, HeightSegs, Radial Segments/Sides, WidthSegs, LengthSegs 
--					Helix Turns, Star Points, Text Size...
--					(more to come)
--	 Documentation: Visit http://www.gfxcentral.com/bobo/mxs4
--	 Code by:		Borislav Petrov [Boboland]		bobo@email.archlab.tuwien.ac.at
--   Based on the Radius Manipulator code by Scott Morrison from Discreet.
----------------------------------------------------------------------------------------

persistent global geometry_manipulators_enabled
if geometry_manipulators_enabled == undefined then 
(
geometry_manipulators_enabled = #()
for i = 1 to 20 do append geometry_manipulators_enabled true 
)

plugin simpleManipulator heightManip
    name:"HeightManip"
    invisible:true
(

    -- Get the green and red colors for the gizmo from the Custom Colors database
    local greenColor =  colorMan.getColor #manipulatorsActive
    local redColor =    colorMan.getColor #manipulatorsSelected
	local size = 3
	
    -- This manipulator manipulates any node with a "Height" property:
	-- *Box
	-- *Cone
	-- *Cylinder
	-- *Tube
	-- *Pyramid
	-- *ChamferBox
	-- *ChamferCylinder
	-- *OilTank
	-- *Capsule
	-- *Spindle
	-- *L_Ext
	-- *Gengon
	-- *C-Ext
	-- *RingWave
	-- *Prism
	
    on canManipulate target return ((hasProperty target #height) and geometry_manipulators_enabled[1])

    -- Create the manipulator gizmo.
    -- This is called initially and whenever the manipulator target changes
	
    on updateGizmos do
    (
        -- Clear the current gizmo cache
        this.clearGizmos()
	
		-- Create a Gizmo Sphere at top of the object with proportional radius
		size = length (node.max - node.min) / 50		
		giz = manip.MakeSphere [0,0,target.height+size*2] size 8

		giz1 = manip.makeGizmoShape()
        giz1.addPoint [0,0,target.height+size]
	    giz1.addPoint [0,0,0] 	
        this.addGizmoShape giz1 gizmoDontHitTest greenColor redColor 
	
        -- Add the Sphere Gizmo to the manipulator
        this.addGizmoMesh giz 0 greenColor redColor  
        this.addGizmoShape giz1 gizmoDontHitTest greenColor redColor 
		
        -- return the ToolTip string
        return node.name + " Height = " + target.height as string 
    )

    -- mouseMove is called on every mouse move when dragging the manip
    -- It needs to convert the mouse position 'm' into a new value for the height
	
    on mouseMove m which do
    (
        -- manip.makePlaneFromNormal takes a normal vector and a point
        -- and creates a plane passing through the point with the given normal
		
		-- we create two planes - one through the X and one through the Y axis,
		-- both through the origin of the object.
		
        local plane1 = manip.makePlaneFromNormal x_axis [0, 0, 0]
        local plane2 = manip.makePlaneFromNormal y_axis [0, 0, 0]
		
		-- Define the projected point
		
        local projectedPoint = [0,0,0]

        -- Compute the hit-ray in local coordinates
        viewRay = this.getLocalViewRay m

        -- Figure out which of the two planes is most "square" to the view ray
        local pl = plane1.mostOrthogonal viewRay plane2

        -- Intersect the plane with the view ray
        res = pl.intersect viewRay &projectedPoint
		
        -- If the intersection worked, set the height
        if (res) then target.height = (projectedPoint.z-size*2.0) 
    )
)


plugin simpleManipulator widthManip
    name:"WidthManip"
    invisible:true
(
    -- Get the green and red colors for the gizmo from the Custom Colors database
    local greenColor =  colorMan.getColor #manipulatorsActive
    local redColor =    colorMan.getColor #manipulatorsSelected
	local node_height = 0
	local size = 3
	
    -- This manipulator manipulates any node with a "Width" property
	-- *Box
	-- *Pyramid
	-- *Plane
	-- *ChamferBox
	-- *QuadPatch
	-- *TriPatch
	
	
    on canManipulate target return ((hasProperty target #width) and (findstring((classof target)as string) "manip" == undefined) and geometry_manipulators_enabled[2])

    -- Create the manipulator gizmo.
    -- This is called initially and whenever the manipulator target changes
	
    on updateGizmos do
    (
        -- Clear the current gizmo cache
        this.clearGizmos()

	-- If there is a height property, use it, if not, use zero
	
		if not(hasProperty target #height) then
			node_height = 0
		else
			node_height = target.height/2
	
		-- Create Gizmo Spheres at the sides of the object with radius based on the total object size
		size = length (node.max - node.min) / 50
		giz1 = manip.MakeSphere [target.width/2+size*2,0,node_height] size 8
		giz2 = manip.MakeSphere [-(target.width/2+size*2),0,node_height] size 8

		giz3 = manip.makeGizmoShape()
        giz3.addPoint [target.width/2+size,0,node_height]
	    giz3.addPoint [-(target.width/2+size),0,node_height]	

	
        -- Add the Sphere Gizmo to the manipulator
        this.addGizmoMesh giz1 0 greenColor redColor  
        this.addGizmoMesh giz2 0 greenColor redColor  
        this.addGizmoShape giz3 gizmoDontHitTest greenColor redColor 
		
        -- return the ToolTip string
        return node.name + " Width = " + target.width as string 
    )

    -- mouseMove is called on every mouse move when dragging the manip
    -- It needs to convert the mouse position 'm' into a new value for the height
	
    on mouseMove m which do
    (
        -- manip.makePlaneFromNormal takes a normal vector and a point
        -- and creates a plane passing through the point with the given normal
		
		-- we create two planes - one through the X and one through the Y axis,
		-- both through the origin of the object.

        local plane1 = manip.makePlaneFromNormal y_axis [target.width/2+size,0,node_height/2]
        local plane2 = manip.makePlaneFromNormal z_axis [target.width/2+size,0,node_height/2]
		
		-- Define the projected point
        local projectedPoint = [0,0,0]

        -- Compute the hit-ray in local coordinates
        viewRay = this.getLocalViewRay m

        -- Figure out which of the two planes is most "square" to the view ray
        local pl = plane1.mostOrthogonal viewRay plane2

        -- Intersect the plane with the view ray
        res = pl.intersect viewRay &projectedPoint
		
        -- If the intersection worked, set the height
        if (res) then target.width = (abs projectedPoint.x*2-size*2) 
    )
)


plugin simpleManipulator lengthManip
    name:"LengthManip"
    invisible:true
(
    -- Get the green and red colors for the gizmo from the Custom Colors database
    local greenColor =  colorMan.getColor #manipulatorsActive
    local redColor =    colorMan.getColor #manipulatorsSelected
	local node_height = 0
	local size = 3
	local length_mode = 1
	local length_value = 0
	
    -- This manipulator manipulates any node with a "Length" or "Depth" property
	-- *Box
	-- *Pyramid (Depth)
	-- *Plane
	-- *ChamferBox
	-- *QuadPatch
	-- *TriPatch
	
    on canManipulate target return (  ((hasProperty target #length) or (hasProperty target #depth)) and geometry_manipulators_enabled[3])

    -- Create the manipulator gizmo.
    -- This is called initially and whenever the manipulator target changes
	
    on updateGizmos do
    (
        -- Clear the current gizmo cache
        this.clearGizmos()

		-- If there is a height property, use it, if not, use zero
	
		if not (hasProperty target #height) then
			node_height = 0
		else
			node_height = target.height/2
	
		-- Create Gizmo Spheres at the sides of the object with radius based on the box's total size
		-- Use the Length or Depth respectively to place it 
		
		size = length (node.max - node.min) / 50
		if (hasProperty target #depth) then
			(
			length_value = target.depth
			length_mode = 2
			)
		else
			(
			length_value = target.length
			length_mode = 1
			)
		
		giz1 = manip.MakeSphere [0,length_value/2+size*2,node_height] size 8
		giz2 = manip.MakeSphere [0,-(length_value/2+size*2),node_height] size 8
		
		giz3 = manip.makeGizmoShape()
        giz3.addPoint [0,length_value/2+size,node_height]
	    giz3.addPoint [0,-(length_value/2+size),node_height]
	
        -- Add the Sphere Gizmo to the manipulator
        this.addGizmoMesh giz1 0 greenColor redColor  
        this.addGizmoMesh giz2 0 greenColor redColor  
        this.addGizmoShape giz3 gizmoDontHitTest greenColor redColor 		
		
        -- return the ToolTip string
		case length_mode of
		(
        1: return node.name + " Length = " + length_value as string 
        2: return node.name + " Depth = " + length_value as string 
		)
    )

    -- mouseMove is called on every mouse move when dragging the manip
    -- It needs to convert the mouse position 'm' into a new value for the height
	
    on mouseMove m which do
    (
        -- manip.makePlaneFromNormal takes a normal vector and a point
        -- and creates a plane passing through the point with the given normal
		
		-- we create two planes - one through the X and one through the Y axis,
		-- both through the origin of the object.

        local plane1 = manip.makePlaneFromNormal x_axis [0,length_value/2+size,node_height/2]
        local plane2 = manip.makePlaneFromNormal z_axis [0,0,node_height/2]
		
		-- Define the projected point
        local projectedPoint = [0,0,0]

        -- Compute the hit-ray in local coordinates
        viewRay = this.getLocalViewRay m

        -- Figure out which of the two planes is most "square" to the view ray
        local pl = plane1.mostOrthogonal viewRay plane2

        -- Intersect the plane with the view ray
        res = pl.intersect viewRay &projectedPoint
		
        -- If the intersection worked, set the height
        if (res) then 
			case length_mode of
			(
			1: target.length = (abs projectedPoint.y*2-size*2) 
			2: target.depth = (abs projectedPoint.y*2-size*2) 
			)
    )
)


plugin simpleManipulator radius1Manip
    name:"Radius1Manip"
    invisible:true
(
    -- Create the green and red colors for the gizmo
    local greenColor =  colorMan.getColor #manipulatorsActive
    local redColor =    colorMan.getColor #manipulatorsSelected

    -- This manipulator manipulates any node with a "radius1" property
    on canManipulate target return (hasProperty target #radius1) and geometry_manipulators_enabled[4]

    -- Create the manipulator gizmo.
    -- This is called initially and whenever the manipulator target changes
    on updateGizmos do
    (
        -- Clear the current gizmo cache
        this.clearGizmos()
        
        -- Set the radius of circle gizmo a little bigger than the target radius
        giz = manip.makeCircle [0,0,0] (target.radius1 * 1.01) 28

        -- Add the circle to the manipulator
        this.addGizmoShape giz 0 greenColor redColor  

        -- return the ToolTip string
        return node.name + " Radius 1 = " + target.radius1 as string 
    )

    -- mouseMove is called on every mouse move when dragging the manip
    -- It needs to convert the mouse position 'm' into a new value for the radius
    on mouseMove m which do
    (
        -- Create the XY plane.   
        -- manip.makePlaneFromNormal takes a normal vector and a point
        -- and creates a plane passing through the point with the given normal
        local pl = manip.makePlaneFromNormal z_axis [0, 0, 0],
        projectedPoint = [0,0,0]

        -- Compute the hit-ray in local coordinates
        viewRay = this.getLocalViewRay m

        -- Intersect the plane with the view ray
        res = pl.intersect viewRay &projectedPoint
    
        -- If the intersection worked, set the radius
        if (res) then target.radius1 = (length projectedPoint) / 1.01
    )
)


plugin simpleManipulator radius2Manip
    name:"Radius2Manip"
    invisible:true
(
    -- Create the green and red colors for the gizmo
    local greenColor =  colorMan.getColor #manipulatorsActive
    local redColor =    colorMan.getColor #manipulatorsSelected
	
	local node_height = 0
	
    -- This manipulator manipulates any node with a "radius2" property:
	-- *Conus
	-- *Tube
	-- *Torus
	-- *TorusKnot
	
    on canManipulate target return (hasProperty target #radius2) and geometry_manipulators_enabled[4]

    -- Create the manipulator gizmo.
    -- This is called initially and whenever the manipulator target changes
    on updateGizmos do
    (
        -- Clear the current gizmo cache
        this.clearGizmos()
		
		-- If there is a height property, use it, if not (like in Torus), use zero
	
		if not (hasProperty target #height) then
			node_height = 0
		else
			node_height = target.height
			
        -- Set the radius of circle gizmo a little bigger than the target radius

		if classof node == Torus then 	
			(
	        giz = manip.makeCircle [0,target.radius1,node_height] (target.radius2 * 1.01) 28
			my_tm = matrixFromNormal [0,1,0]
			giz.Transform my_tm
			size = length (node.max - node.min) / 100
			giz2 = manip.MakeSphere [target.radius1-target.radius2-size,0,node_height] size 8
			giz3 = manip.MakeSphere [target.radius1+target.radius2+size,0,node_height] size 8
			
	        this.addGizmoShape giz GizmoDontHitTest greenColor redColor  
	        this.addGizmoMesh giz2 0 greenColor redColor  
	        this.addGizmoMesh giz3 0 greenColor redColor  
			)
			else
			(
	        giz = manip.makeCircle [0,0,node_height] (target.radius2 * 1.01) 28
	        this.addGizmoShape giz 0 greenColor redColor  
			)

        -- Add the circle to the manipulator

        -- return the ToolTip string
        return node.name + " Radius 2 = " + target.radius2 as string 
    )

    -- mouseMove is called on every mouse move when dragging the manip
    -- It needs to convert the mouse position 'm' into a new value for the radius
    on mouseMove m which do
    (
	
		if classof node == Torus then 	
		(
        local p1 = manip.makePlaneFromNormal y_axis [0, 0, 0]
        local p2 = manip.makePlaneFromNormal z_axis [0, 0, 0]
		
        local projectedPoint = [0,0,0]
        viewRay = this.getLocalViewRay m
        local plane = p1.mostOrthogonal viewRay p2
		
        res = plane.intersect viewRay &projectedPoint
        if (res) then target.radius2 = abs(projectedPoint.x-target.radius1) / 1.01
		)
		else
		(

        local p1 = manip.makePlaneFromNormal z_axis [0, 0, node_height]
        local projectedPoint = [0,0,node_height]
        viewRay = this.getLocalViewRay m
        res = p1.intersect viewRay &projectedPoint
        if (res) then 
			(
			target.radius2 = (sqrt(projectedPoint.x^2+projectedPoint.y^2)) / 1.01
			)
		)
		
    )
)


plugin simpleManipulator capheightManip
    name:"CapheightManip"
    invisible:true
(
    -- Create the green and red colors for the gizmo
    local greenColor =  colorMan.getColor #manipulatorsActive
    local redColor =    colorMan.getColor #manipulatorsSelected
	
	local cap_value = 0
	local cap_mode = 1
	local size = 3
	
    -- This manipulator manipulates any node with a "CapHeight" or "Fillet" property
	-- *Spindle
	-- *OilTank
	-- *ChamferCylinder
	
    on canManipulate target return ((hasProperty target #fillet) or (hasProperty target #cap_height)) and geometry_manipulators_enabled[9]

    -- Create the manipulator gizmo.
    -- This is called initially and whenever the manipulator target changes
    on updateGizmos do
    (
        -- Clear the current gizmo cache
        this.clearGizmos()
		
		if not (hasProperty target #fillet) then
			(
			cap_value = target.cap_height
			cap_mode = 1
			)
		else
			(
			cap_value = target.fillet
			cap_mode = 2
			)		
		
		if (hasProperty target #radius) then
			(
			radius1_value = target.radius
			radius2_value = target.radius
			)
		else
			(
			radius1_value = target.width/2
			radius2_value = target.length/2
			)		
		
        
        -- Set a Gizmo with size proportional to the manipulated node's size
		size = length (node.max - node.min) / 50
        giz1 = manip.makeBox [radius1_value+size,0,cap_value] size (size*2) size 1 1 1
        giz2 = manip.makeBox [-(radius1_value+size),0,cap_value] size (size*2) size 1 1 1
        giz3 = manip.makeBox [0,radius2_value+size,cap_value] (size*2) size size 1 1 1
        giz4 = manip.makeBox [0,-(radius2_value+size),cap_value] (size*2) size size 1 1 1

        -- Add the circle to the manipulator
		this.addGizmoMesh giz1 0 greenColor redColor  
        this.addGizmoMesh giz2 0 greenColor redColor  
        this.addGizmoMesh giz3 0 greenColor redColor  
        this.addGizmoMesh giz4 0 greenColor redColor  

        -- return the ToolTip string
		case cap_mode of
		(
        1: return node.name + " Cap Height = " + cap_value as string 
        2: return node.name + " Fillet = " + cap_value as string 
		)
    )

    -- mouseMove is called on every mouse move when dragging the manip
    -- It needs to convert the mouse position 'm' into a new value for the radius
    on mouseMove m which do
    (
        -- Create the XY plane.   
        -- manip.makePlaneFromNormal takes a normal vector and a point
        -- and creates a plane passing through the point with the given normal
		
        local p1 = manip.makePlaneFromNormal x_axis [0, 0, 0]
        local p2 = manip.makePlaneFromNormal y_axis [0, 0, 0]
		
        local projectedPoint = [0,0,0]

        -- Compute the hit-ray in local coordinates
        viewRay = this.getLocalViewRay m

        -- Figure out which of the two planes is most "square" to the view ray
        local planel = p1.mostOrthogonal viewRay p2
		
        -- Intersect the plane with the view ray
        res = planel.intersect viewRay &projectedPoint
    
        -- If the intersection worked, set the radius
        if res and projectedPoint.z >0 then 
			(
			case cap_mode of
				(
				1: target.cap_height = (projectedPoint.z)
				2: target.fillet = (projectedPoint.z)
				)
			)
    )
)



plugin simpleManipulator heightSegsManip
    name:"HeightSegsManip"
    invisible:true
(
    -- Get the green and red colors for the gizmo from the Custom Colors database
    local greenColor =  colorMan.getColor #manipulatorsActive
    local redColor =    colorMan.getColor #manipulatorsSelected
	local size = 3
	local hsegs_value = 1
	local hsegs_mode = 2	
	
    -- This manipulator manipulates any node with a "HeightSegs" or "Height_Segments" property:
	-- *Box
	-- *Cone
	-- *Cylinder
	-- *Tube
	-- *Pyramid
	-- *ChamferBox
	-- *ChamferCylinder
	-- *OilTank
	-- *Capsule
	-- *Spindle
	-- *L_Ext
	-- *Gengon
	-- *C-Ext
	-- *Prism
	
    on canManipulate target return (((hasProperty target #heightsegs)) or (hasProperty target #height_segments)) and (hasProperty target #height) and geometry_manipulators_enabled[6]

    -- Create the manipulator gizmo.
    -- This is called initially and whenever the manipulator target changes
	
    on updateGizmos do
    (
        -- Clear the current gizmo cache
        this.clearGizmos()
		
		-- Check for the exact property name - some primitives use "heightsegs", other "Height_Segments".
		if (hasProperty target #heightsegs) then
			(
			hsegs_value = target.heightsegs
			hsegs_mode = 1
			)
		else
			(
			hsegs_value = target.Height_Segments
			hsegs_mode = 2
			)		
		
	
		-- Create a Gizmo in the corner with proportional size
		size = length (node.max - node.min) / 50
		giz_z_pos = target.height/50* hsegs_value 
		giz1 = manip.MakeBox [(node.max.x-node.min.x+size)/2,0,giz_z_pos] size size (size*2) 1 1 3
		giz2 = manip.MakeBox [-(node.max.x-node.min.x+size)/2,0,giz_z_pos] size size (size*2) 1 1 3
	
        -- Add the Gizmo to the manipulator
        this.addGizmoMesh giz1 0 greenColor redColor  
        this.addGizmoMesh giz2 0 greenColor redColor  
		
        -- return the ToolTip string
        return node.name + " Height Segments = " + hsegs_value as string 
    )

    -- mouseMove is called on every mouse move when dragging the manip
    -- It needs to convert the mouse position 'm' into a new value for the height
	
    on mouseMove m which do
    (
        -- manip.makePlaneFromNormal takes a normal vector and a point
        -- and creates a plane passing through the point with the given normal
		-- we create two planes - one through the X and one through the Y axis,
		-- both through the origin of the object.

        local plane1 = manip.makePlaneFromNormal x_axis [0,0,0]
        local plane2 = manip.makePlaneFromNormal y_axis [0,0,0]
		
		-- Define the projected point
        local projectedPoint = [0,0,0]

        -- Compute the hit-ray in local coordinates
        viewRay = this.getLocalViewRay m

        -- Figure out which of the two planes is most "square" to the view ray
        local pl = plane1.mostOrthogonal viewRay plane2

        -- Intersect the plane with the view ray
        res = pl.intersect viewRay &projectedPoint
		
        -- If the intersection worked and heightsegs is in valid range, set the heightsegs
		heightsegs_value = projectedPoint.z/((node.max.z-node.min.z)/50)		
		if heightsegs_value > 0 and heightsegs_value < 201 and res then 
			(
			case hsegs_mode of
			(
			1: target.heightsegs = heightsegs_value
			2: target.Height_Segments = heightsegs_value
			)
			)
    )
)


plugin simpleManipulator RadiusSegsManip
    name:"RadiusSegsManip"
    invisible:true
(
    -- Get the green and red colors for the gizmo from the Custom Colors database
    local greenColor =  colorMan.getColor #manipulatorsActive
    local redColor =    colorMan.getColor #manipulatorsSelected
	local multi_factor = 0.9
	local segs_mode = 1
	local node_segs = 1
	
    -- This manipulator manipulates any modifier with an "Segs" property:
	--*Sphere
	--*Geosphere
	--*Cylinder
	--*Tube
	--*Torus
	--*Teapot
	--*TorusKnot
	--*ChamferCyl
	--*OilTank
	--*Capsule
	--*Spindle
	--*Gengon
	--*RingWave
	
    on canManipulate target return
		(
		((hasProperty target #segs) or (hasProperty target #sides) or (hasProperty target #'ring segments') or (hasProperty target #segments)) \
		and ((hasProperty target #radius) or (hasProperty target #radius1) or (hasProperty target #'max diameter')) and geometry_manipulators_enabled[5] and superclassof target != Shape
		)

    on updateGizmos do
    (
	
		case (classof target) of
		(
		Sphere: multi_factor = 0.9
		Geosphere: multi_factor = 6
		Cylinder: multi_factor = 0.9
		Tube: multi_factor = 0.9
		Torus: multi_factor = 0.9
		Teapot: multi_factor = 8
		TorusKnot: multi_factor = 10
		ChamferBox: multi_factor = 2
		OilTank: multi_factor = 2
		Capsule: multi_factor = 2
		Spindle: multi_factor = 2
		
		)


		if (hasProperty target #sides) then			
			(
			node_segs = target.sides
			segs_mode = 2
			)
		
		if (hasProperty target #segs) then			
			(
			node_segs = target.segs 
			segs_mode = 1
			)

		if (hasProperty target #segments) then			
			(
			node_segs = target.segments
			segs_mode = 4
			)
			
			
		if (hasProperty target #'ring segments') then
			(
			node_segs = target.sides
			segs_mode = 3
			)
			
        this.clearGizmos()
		
		local size1 = length (node.max - node.min) / 50

		if (hasProperty target #radius) then size2 = target.radius*1.2
		if (hasProperty target #'max diameter') then size2 = target.radius*1.2
		if (hasProperty target #radius1) then size2 = target.radius1*1.2
			
		
		giz2 = manip.MakeCircle [0,0,0] size2 24
		giz1 = manip.MakeSphere [size2*cos(node_segs*multi_factor),size2*sin(node_segs*multi_factor),0] size1 8
		giz4 = manip.MakeSphere [size2*cos(node_segs*multi_factor+180),size2*sin(node_segs*multi_factor+180),0] size1 8
		
        giz3 = manip.makeGizmoShape()
        giz3.addPoint [size2*cos(node_segs*multi_factor),size2*sin(node_segs*multi_factor),0] 
        giz3.addPoint [size2*cos(node_segs*multi_factor+180),size2*sin(node_segs*multi_factor+180),0]
 
        this.addGizmoMesh giz1 0 greenColor redColor  
        this.addGizmoShape giz2 gizmoDontHitTest greenColor redColor  
        this.addGizmoShape giz3 gizmoDontHitTest greenColor redColor  
        this.addGizmoMesh giz4 0 greenColor redColor  
		
		case segs_mode of
		(
        1: return node.name + " Segs = " + node_segs as string 
        2: return node.name + " Sides = " + node_segs as string 
        3: return node.name + " Sides = " + node_segs as string 
        4: return node.name + " Segments = " + node_segs as string 
		)
    )

    on mouseMove m which do
    (
		
		local p1 = manip.makePlaneFromNormal z_axis [0, 0, 0]
        local projectedPoint = [0,0,0]
        viewRay = this.getLocalViewRay m
        res = p1.intersect viewRay &projectedPoint
        if (res) then 
			(
			tangens_value = projectedPoint.x/projectedPoint.y
			if projectedPoint.x >= 0 and projectedPoint.y > 0 then result = 90 - (atan (tangens_value))
			if projectedPoint.x <= 0 and projectedPoint.y > 0 then result = 90 - (atan (tangens_value))
			if projectedPoint.x <= 0 and projectedPoint.y < 0 then result = 90 - (atan (tangens_value))
			
			if projectedPoint.x >= 0 and projectedPoint.y < 0 then result = 90 - (atan (tangens_value))
			if result == undefined then result = 0
			case segs_mode of
				(
				1: target.segs  = abs result / multi_factor
				2: target.sides = abs result / multi_factor
				3: target.sides = abs result / multi_factor
				4: target.segments = abs result / multi_factor
				)
			)
    )
)


plugin simpleManipulator widthSegsManip
    name:"WidthSegsManip"
    invisible:true
(
    -- Get the green and red colors for the gizmo from the Custom Colors database
    local greenColor =  colorMan.getColor #manipulatorsActive
    local redColor =    colorMan.getColor #manipulatorsSelected
	local size = 3
	local wsegs_mode = 1
	local wsegs_value = 1
	
    -- This manipulator manipulates any node with a "WidthSegs" " property:
	-- *Box
	-- *Pyramid
	-- *Plane
	-- *ChamferBox
	-- *QuadPatch
	
    on canManipulate target return (((hasProperty target #widthsegs) ) or (hasProperty target #width_segments) ) and (hasProperty target #width) and geometry_manipulators_enabled[7]

    on updateGizmos do
    (
        this.clearGizmos()
		
		if (hasProperty target #widthsegs) then
			(
			wsegs_value = target.widthsegs
			wsegs_mode = 1
			)
		else
			(
			wsegs_value = target.Width_Segments
			wsegs_mode = 2
			)		

		if (hasProperty target #length) then
			(
			giz_l_pos = target.length
			)
		else
			(
			giz_l_pos = target.depth
			)		
	
		size = length (node.max - node.min) / 50
		giz_w_pos = target.width/100 * wsegs_value 
		giz1 = manip.MakeBox [giz_w_pos-target.width/2,(giz_l_pos+size)/2,0]  size (size*2) size 1 3 1
		giz2 = manip.MakeBox [giz_w_pos-target.width/2,-(giz_l_pos+size)/2,0] size (size*2) size 1 3 1
        this.addGizmoMesh giz1 0 greenColor redColor  
        this.addGizmoMesh giz2 0 greenColor redColor  
        return node.name + " Width Segments = " + wsegs_value as string 
    )

    on mouseMove m which do
    (
        local plane = manip.makePlaneFromNormal z_axis [0,0,0]
        local projectedPoint = [0,0,0]
        viewRay = this.getLocalViewRay m
        res = plane.intersect viewRay &projectedPoint
		
		widthsegs_value = (projectedPoint.x+target.width/2)/((target.width)/100)		
		if widthsegs_value > 0 and widthsegs_value < 201 and res then 
			(
			case wsegs_mode of
			(
			1: target.widthsegs = widthsegs_value 
			2: target.Width_Segments = widthsegs_value 
			)
			)
    )
)


plugin simpleManipulator lengthSegsManip
    name:"LengthSegsManip"
    invisible:true
(
    -- Get the green and red colors for the gizmo from the Custom Colors database
    local greenColor =  colorMan.getColor #manipulatorsActive
    local redColor =    colorMan.getColor #manipulatorsSelected
	local size = 3
	local lsegs_mode = 1
	local lsegs_value = 1
	local node_length = 0
	
    -- This manipulator manipulates any node with a "LengthSegs" or "DepthSegs" property:
	-- *Box
	-- *Pyramid
	-- *Plane
	-- *ChamferBox
	-- *QuadPatch
	
    on canManipulate target return (((hasProperty target #lengthsegs) ) or (hasProperty target #length_segments)  or (hasProperty target #depthsegs) ) and ((hasProperty target #length)  or (hasProperty target #depth) ) and geometry_manipulators_enabled[8]

    -- Create the manipulator gizmo.
    -- This is called initially and whenever the manipulator target changes
	
    on updateGizmos do
    (
        this.clearGizmos()
		
		if (hasProperty target #lengthsegs) then
			(
			lsegs_value = target.lengthsegs
			lsegs_mode = 1
			)
		if (hasProperty target #length_segments) then
			(
			lsegs_value = target.Length_Segments
			lsegs_mode = 2
			)		
		if (hasProperty target #depthsegs) then
			(
			lsegs_value = target.depthsegs
			lsegs_mode = 3
			)		

		if (hasProperty target #length) then
			(
			giz_l_pos = target.length/100 * lsegs_value 
			node_length = target.length
			)
		else
			(
			giz_l_pos = target.depth/100 * lsegs_value 
			node_length = target.depth
			)		
	
		size = length (node.max - node.min) / 50
		giz_w_pos = target.width
		giz1 = manip.MakeBox [(giz_w_pos+size)/2,giz_l_pos-node_length/2,0]  (size*2) size size  3 1 1
		giz2 = manip.MakeBox [-(giz_w_pos+size)/2,giz_l_pos-node_length/2,0] (size*2) size size  3 1 1
	
        this.addGizmoMesh giz1 0 greenColor redColor  
        this.addGizmoMesh giz2 0 greenColor redColor  
		
		if lsegs_mode == 3 then
    	    return node.name + " Depth Segments = " + lsegs_value as string 
		else
	        return node.name + " Length Segments = " + lsegs_value as string 
    )

    on mouseMove m which do
    (
        local plane = manip.makePlaneFromNormal z_axis [0,0,0]
        local projectedPoint = [0,0,0]
        viewRay = this.getLocalViewRay m
        res = plane.intersect viewRay &projectedPoint
		lengthsegs_value = (projectedPoint.y+node_length/2)/((node_length)/100)		
		if lengthsegs_value > 0 and lengthsegs_value < 201 and res then 
			(
			case lsegs_mode of
			(
			1: target.lengthsegs = lengthsegs_value 
			2: target.Length_Segments = lengthsegs_value 
			3: target.depthsegs = lengthsegs_value 
			)
			)
    )
)

plugin simpleManipulator TurnsSegsManip
    name:"TurnsSegsManip"
    invisible:true
(
    -- Get the green and red colors for the gizmo from the Custom Colors database
    local greenColor =  colorMan.getColor #manipulatorsActive
    local redColor =    colorMan.getColor #manipulatorsSelected
	local turns_mode = 1

    -- This manipulator manipulates the "Turns" property of Helix Shapes:
	
    on canManipulate target return
		(
		((classof target == Star) or (classof target == Helix)) and geometry_manipulators_enabled[10]
		--(((findItem (getPropNames target) #turns) > 0) or (	(findItem (getPropNames target) #numpoints) > 0 and ) and geometry_manipulators_enabled[10]
		)

    on updateGizmos do
    (
	
        this.clearGizmos()


		local size2 = target.radius1*1.2 
		local size1 = target.radius1/15
		
		if classof target == Helix then
		(
		node_turns = target.turns*10
		turns_mode = 1
		)
		if classof target == Star then
		(
		node_turns = target.numpoints*1.8
		turns_mode = 2
		)
		
		giz2 = manip.MakeCircle [0,0,0] size2 24
		giz1 = manip.MakeSphere [size2*cos(node_turns),size2*sin(node_turns),0] size1 8
		giz4 = manip.MakeSphere [size2*cos(node_turns+180),size2*sin(node_turns+180),0] size1 8
		
        giz3 = manip.makeGizmoShape()
        giz3.addPoint [size2*cos(node_turns),size2*sin(node_turns),0] 
        giz3.addPoint [size2*cos(node_turns+180),size2*sin(node_turns+180),0]
 
        this.addGizmoMesh giz1 0 greenColor redColor  
        this.addGizmoShape giz2 gizmoDontHitTest greenColor redColor  
        this.addGizmoShape giz3 gizmoDontHitTest greenColor redColor  
        this.addGizmoMesh giz4 0 greenColor redColor  
		
		case turns_mode of
		(
        1: return node.name + " Turns = " + target.turns as string 
		2: return node.name + " Points = " + target.numpoints as string 
		)
    )

    on mouseMove m which do
    (
		
		local p1 = manip.makePlaneFromNormal z_axis [0, 0, 0]
        local projectedPoint = [0,0,0]
        viewRay = this.getLocalViewRay m
        res = p1.intersect viewRay &projectedPoint
        if res then 
			(
			tangens_value = projectedPoint.x/projectedPoint.y
			if projectedPoint.x >= 0 and projectedPoint.y > 0 then result = 90 - (atan (tangens_value))
			if projectedPoint.x <= 0 and projectedPoint.y > 0 then result = 90 - (atan (tangens_value))
			if projectedPoint.x <= 0 and projectedPoint.y < 0 then result = 90 - (atan (tangens_value))
			
			if projectedPoint.x >= 0 and projectedPoint.y < 0 then result = 90 - (atan (tangens_value))
			if result == undefined then result = 0

			case turns_mode of
			(
			1: target.turns = abs result / 10
			2: target.numpoints  = abs result /1.8
			)
			
			)
    )
)



plugin simpleManipulator TextSizeSegsManip
    name:"TextSizeSegsManip"
    invisible:true
(
    -- Get the green and red colors for the gizmo from the Custom Colors database
    local greenColor =  colorMan.getColor #manipulatorsActive
    local redColor =    colorMan.getColor #manipulatorsSelected
	local size = 3

    -- This manipulator manipulates the Size Property of the Text Shape
    on canManipulate target return classof target == Text and geometry_manipulators_enabled[1]

    on updateGizmos do
    (
        this.clearGizmos()
		size = 1 + length (node.max - node.min) / 100
		giz_w_pos = target.size / 1.5
		giz1 = manip.MakeSphere [0,giz_w_pos,0] size 8
        this.addGizmoMesh giz1 0 greenColor redColor  
   	    return node.name + " Size = " + target.size as string 
    )

    on mouseMove m which do
    (
        local plane = manip.makePlaneFromNormal z_axis [0,0,0]
        local projectedPoint = [0,0,0]
        viewRay = this.getLocalViewRay m
        res = plane.intersect viewRay &projectedPoint
		if res and (projectedPoint.y)*1.5 > 0 then target.size = (projectedPoint.y)*1.5
    )
)

